/*
    This file is part of Frozen Bubble for S60.

    Frozen Bubble for S60 is free software; you can redistribute
    it and/or modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2 of
    the License, or (at your option) any later version.

    Frozen Bubble for S60 is distributed in the hope that it
    will be useful, but WITHOUT ANY WARRANTY; without even the implied
    warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
    See the GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Frozen Bubble for S60; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __CSNDMIXER_H__
#define __CSNDMIXER_H__

#include <E32Base.h>
#include "TSample.h"
#include "TAudioShared.h"

// CSndMixer is the main class for user to produce sound.
class CSndMixer : public CBase {
public:
  /// Two-phased constructor
  static CSndMixer* NewL();

  /// Default destructor
  ~CSndMixer();

private:

  /// Default constructor
  CSndMixer();

  /// Second phase constructor
  void ConstructL();

public:

  /// Pauses mixer
  void Pause();

  /// Resumes mixer
  void Resume();

  /// Plays a sample with given parameters
  /// @param aSample sample to play
  /// @param aChannel channel 0-15 to play the sample on
  /// @param aVolume volume to play sample with, default is 256
  void Play( const TSample& aSample, TInt aChannel, TInt aFrequency, TInt aVolume = 256 );

  /** 
   * Starts music in mikmod
   * @param name of music to load
   */
  void PlayMusic(const char* aFileName);

  /**
   * Pause the module playback
   * @param true if playback should be paused
   */
  void PauseMusic(TBool aPauseState);

  /// Stops sample from playing in given channel
  /// @param aChannel channel to play sample in
  void Stop( TInt aChannel );

  /// Sets the main volume
  /// @param aVolume new main volume
  void SetVolume( TInt aVolume );

  /// returns main volume
  /// @return main volume
  TInt Volume();

private:

  void SendCmd( TMixerCmd aCmd );

private:

  TAudioShared  iShared;        // shared data with mixer thread
  RThread       iMixerThread;   // handle to mixer thread
  TSample       iEmptySample;   // empty sample, used to stop channel
  TBool         iPaused;        // paused flag
};
  
#endif


