/*
    This file is part of Frozen Bubble for S60.

    Frozen Bubble for S60 is free software; you can redistribute
    it and/or modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2 of
    the License, or (at your option) any later version.

    Frozen Bubble for S60 is distributed in the hope that it
    will be useful, but WITHOUT ANY WARRANTY; without even the implied
    warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
    See the GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Frozen Bubble for S60; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "CWavLoader.h"
#include <e32svr.h>
#include <f32file.h>
#include <eikenv.h>
#ifdef UIQ3
#else
	#include <AknUtils.h>
#endif


CWavLoader* CWavLoader::NewL() {
  CWavLoader* self = NewLC();
  CleanupStack::Pop( self );
  return self;
}


CWavLoader* CWavLoader::NewLC() {
  CWavLoader* self = new( ELeave )CWavLoader();
  CleanupStack::PushL( self );
  self->ConstructL();
  return self;
}


CWavLoader::~CWavLoader() {
}


CWavLoader::CWavLoader() {
}


void CWavLoader::ConstructL() {
}


TSample CWavLoader::LoadL( const TFileName& aFileName ) {
  TFileName name( aFileName );
  
//#ifndef __SERIES60_30__
//  CompleteWithAppPath( name );
  // windows build wants to install apps on rom drive ( "z:\" )
  // but we want the data from "c:\" instead
//#ifdef __WINS__
//  name[ 0 ] = 'C';
//#endif  // __WINS__
//#endif  // !__SERIES60_30__
  
  RFile file;
  CleanupClosePushL( file );
  User::LeaveIfError( file.Open(CEikonEnv::Static()->FsSession(), name, EFileStream | EFileRead ) );
  TInt size;
  User::LeaveIfError( file.Size( size ) );

  TUint8* tbuf = new( ELeave )TUint8[ size ];
  CleanupStack::PushL(tbuf);
  
  TPtr8 ptr( (TUint8*)tbuf, size );
  User::LeaveIfError( file.Read( ptr ) ); 
  
  TInt16* buf = new( ELeave )TInt16[ size ];
  TInt i;
  for( i=0; i<size; i++ )
    {
    TInt v = tbuf[ i ];
    v -= 128;
    buf[ i ] = ( TInt16 )( v * 256 );
    }

  CleanupStack::PopAndDestroy( 2 ); // tbuf file
  
  TSample sample;
  sample.iData = buf;
  sample.iLength = size;
  sample.iRepStart = 0;
  sample.iRepEnd = 0;
  
  return sample;
}
  
