/*
    This file is part of Frozen Bubble for S60.

    Frozen Bubble for S60 is free software; you can redistribute
    it and/or modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2 of
    the License, or (at your option) any later version.

    Frozen Bubble for S60 is distributed in the hope that it
    will be useful, but WITHOUT ANY WARRANTY; without even the implied
    warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
    See the GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Frozen Bubble for S60; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// INCLUDE FILES
#include <eikapp.h>
#ifdef UIQ3
#include <qikon.hrh>
#else
#include <avkon.hrh>
#include <aknnotewrappers.h>
#include <stringloader.h>
#include <aknsoundsystem.h>
#endif
#include <FrozenBubble.rsg>
#include <f32file.h>
#include <s32file.h>

#include "FrozenBubble.pan"
#include "FrozenBubbleAppUi.h"
#include "FrozenBubbleAppView.h"
#include "FrozenBubble.hrh"

// ============================ MEMBER FUNCTIONS ===============================


// -----------------------------------------------------------------------------
// CFrozenBubbleAppUi::ConstructL()
// Symbian 2nd phase constructor can leave.
// -----------------------------------------------------------------------------
//
void CFrozenBubbleAppUi::ConstructL() {
  // Initialise app UI with standard value.
  BaseConstructL();

  // Disable key sounds
#ifndef UIQ3
  KeySounds()->PushContextL( R_AVKON_SILENT_SKEY_LIST ); 
#endif

	// Create view object
  iAppView = CFrozenBubbleAppView::NewL( ClientRect() );
#ifdef UIQ3
  AddViewL(*iAppView);
#else
  iAppView->SetMopParent( this );
#endif
  iMainEngine = new (ELeave) CFrozenBubbleMain(this);

  iLancher = CIdle::NewL( CActive::EPriorityIdle );
  iLancher->Start(TCallBack(Start,this));

  TSize iScreenRes = CEikonEnv::Static()->ScreenDevice()->SizeInPixels();
  ptTopLeft.SetXY( (iScreenRes.iWidth-iMainEngine->GetGameScreenSize().iWidth)/2, (iScreenRes.iHeight-iMainEngine->GetGameScreenSize().iHeight)/2 );
}

// -----------------------------------------------------------------------------
// CFrozenBubbleAppUi::CFrozenBubbleAppUi()
// C++ default constructor can NOT contain any code, that might leave.
// -----------------------------------------------------------------------------
//
CFrozenBubbleAppUi::CFrozenBubbleAppUi() {
  // No implementation required
}

// -----------------------------------------------------------------------------
// CFrozenBubbleAppUi::~CFrozenBubbleAppUi()
// Destructor.
// -----------------------------------------------------------------------------
//
CFrozenBubbleAppUi::~CFrozenBubbleAppUi() {
#ifndef UIQ3
  if( iAppView ) {
      delete iAppView;
      iAppView = NULL;
  }
#endif

  if( iMainEngine ){
	  delete iMainEngine;
  	iMainEngine = NULL;
  }
  delete iLancher;
}

// -----------------------------------------------------------------------------
// CFrozenBubbleAppUi::HandleCommandL()
// Takes care of command handling.
// -----------------------------------------------------------------------------
//
void CFrozenBubbleAppUi::HandleCommandL( TInt aCommand ) {
  switch ( aCommand ) {
#ifdef UIQ3
#else
    case EAknSoftkeyOk:
      iMainEngine->ExternalEvent(EVT_SELECT);
      break;
    case EAknSoftkeyBack:
      iMainEngine->ExternalEvent(EVT_ESCAPE);
      break;
#endif
    case EEikCmdExit:
      iMainEngine->ForceQuit();
      Exit();
      break;
    default:
      Panic( EFrozenBubbleUi );
      break;      
  }
}
    
TInt CFrozenBubbleAppUi::Start(TAny* appui){
  CFrozenBubbleAppUi *fbappui = (CFrozenBubbleAppUi*) appui;
  delete fbappui->iLancher;
  fbappui->iLancher = NULL;
  fbappui->iMainEngine->main();
  return false;
}

// ----------------------------------------------------
// CFrozenBubbleAppUi::HandleKeyEventL(
//     const TKeyEvent& aKeyEvent,TEventCode /*aType*/)
// takes care of key event handling
// ----------------------------------------------------
//
TKeyResponse CFrozenBubbleAppUi::HandleKeyEventL(const TKeyEvent& aKeyEvent,TEventCode aType) {
  if(iMainEngine)
    return iMainEngine->DoKeyEvent(aKeyEvent, aType);
  return EKeyWasNotConsumed;
}

void CFrozenBubbleAppUi::FlipBackBuffer(void){
  iAppView->FlipBackBuffer(ptTopLeft, TRect(TPoint(0,0), iMainEngine->GetGameScreenSize()));
}

void CFrozenBubbleAppUi::BitBlt(TInt iBltType, CExtendedBitmap &iBitmap, const TPoint &aPosition){
  switch(iBltType){
    case BLTTYPE_NORMAL: 
      iAppView->BlitToBackBuffer(iBitmap, aPosition);
      break;
    case BLTTYPE_MASKED: 
      iAppView->BlitToBackBufferMasked(iBitmap, aPosition);
      break;
  }
}

void CFrozenBubbleAppUi::BitBlt(TInt iBltType, CExtendedBitmap &iBitmap, const TPoint &aPosition, const TRect &aRect){
  switch(iBltType){
    case BLTTYPE_NORMAL: 
      iAppView->BlitToBackBuffer(iBitmap, aPosition, aRect);
      break;
    case BLTTYPE_MASKED: 
      iAppView->BlitToBackBufferMasked(iBitmap, aPosition, aRect);
      break;
  }
}

void CFrozenBubbleAppUi::BitBltBack(CPrintableExtendedBitmap &iBitmapTarget){
  iAppView->BitBltBack(iBitmapTarget);
}

void CFrozenBubbleAppUi::DoExit(void){
  Exit();
}

void CFrozenBubbleAppUi::HandleForegroundEventL(TBool aForeground){
  if( aForeground ){
    iMainEngine->ExternalEvent(EVT_GOFOREGR);
  } else
    iMainEngine->ExternalEvent(EVT_GOBACKGR);
#ifdef UIQ3
  CQikAppUi::HandleForegroundEventL(aForeground);
#else
  CAknAppUi::HandleForegroundEventL(aForeground);
#endif
}

TBool CFrozenBubbleAppUi::ReadIniFile(TUid iInfo, void *ptr, TUint &size){
  TInt r;
  TBool result = EFalse;
  RDictionaryReadStream rdsIniFile;
  CDictionaryStore *cdIniFile = Application()->OpenIniFileLC(iEikonEnv->FsSession());
  if (cdIniFile->IsPresentL(iInfo)){
    rdsIniFile.OpenLC(*cdIniFile, iInfo);
    TPtr8 buf((TUint8*)ptr, size);
    TRAP(r, rdsIniFile.ReadL(buf));
    CleanupStack::PopAndDestroy(); // rdsIniFile
    result = ETrue;
  }
  CleanupStack::PopAndDestroy( 1 ); //cdIniFile
  return result;
}

TBool CFrozenBubbleAppUi::CheckIniFile(TUid iInfo){
  TBool result = EFalse;
//  RDictionaryReadStream rdsIniFile;
  CDictionaryStore *cdIniFile = Application()->OpenIniFileLC(iEikonEnv->FsSession());
  result = cdIniFile->IsPresentL(iInfo);
  CleanupStack::PopAndDestroy( 1 ); // cdIniFile
  return result;
}

void CFrozenBubbleAppUi::WriteIniFile(TUid iInfo, void *ptr, TUint size){
  TInt r;

  RDictionaryWriteStream rdsIniFile;
  CDictionaryStore *cdIniFile = Application()->OpenIniFileLC(iEikonEnv->FsSession());
  rdsIniFile.AssignLC(*cdIniFile, iInfo);
  TPtr8 buf((TUint8*)ptr, size, size);
  TRAP(r, rdsIniFile.WriteL(buf));
  rdsIniFile.CommitL();
  CleanupStack::PopAndDestroy();
  cdIniFile->CommitL();
  CleanupStack::PopAndDestroy( 1 ); // close file
}

void CFrozenBubbleAppUi::RemoveIniData(TUid iInfo){
//  TInt r;
//  RDictionaryWriteStream rdsIniFile;
  CDictionaryStore *cdIniFile = Application()->OpenIniFileLC(iEikonEnv->FsSession());
  cdIniFile->Remove(iInfo);
  cdIniFile->CommitL();
  CleanupStack::PopAndDestroy( 1 );
}

// -----------------------------------------------------------------------------
//  Called by the framework when the application status pane
//  size is changed.  Passes the new client rectangle to the
//  AppView
// -----------------------------------------------------------------------------
//
void CFrozenBubbleAppUi::HandleStatusPaneSizeChange() {
	iAppView->SetRect( ClientRect() );
} 

CFrozenBubbleMain& CFrozenBubbleAppUi::MainEngine()
{
	return *iMainEngine;
}


// End of File
