/*
    This file is part of Frozen Bubble for S60.

    Frozen Bubble for S60 is free software; you can redistribute
    it and/or modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2 of
    the License, or (at your option) any later version.

    Frozen Bubble for S60 is distributed in the hope that it
    will be useful, but WITHOUT ANY WARRANTY; without even the implied
    warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
    See the GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Frozen Bubble for S60; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __FROZENBUBBLEAPPUI_h__
#define __FROZENBUBBLEAPPUI_h__

// INCLUDES
#ifdef UIQ3
	#include <qikappui.h>
#else
	#include <aknappui.h>
#endif

#include "CFrozenBubbleMain.h"


// FORWARD DECLARATIONS
class CFrozenBubbleAppView;


// CLASS DECLARATION
/**
* CFrozenBubbleAppUi application UI class.
* Interacts with the user through the UI and request message processing
* from the handler class
*/
#ifdef UIQ3
class CFrozenBubbleAppUi : public CQikAppUi, public MFrozenBubbleAction {
#else
class CFrozenBubbleAppUi : public CAknAppUi, public MFrozenBubbleAction {
#endif
public: // Constructors and destructor

    /**
    * ConstructL.
    * 2nd phase constructor.
    */
    void ConstructL();

    /**
    * CFrozenBubbleAppUi.
    * C++ default constructor. This needs to be public due to
    * the way the framework constructs the AppUi
    */
    CFrozenBubbleAppUi();

    /**
    * ~CFrozenBubbleAppUi.
    * Virtual Destructor.
    */
    virtual ~CFrozenBubbleAppUi();

private:  // Functions from base classes

    /**
    * From CEikAppUi, HandleCommandL.
    * Takes care of command handling.
    * @param aCommand Command to be handled.
    */
    void HandleCommandL( TInt aCommand );

    /**
    *  HandleStatusPaneSizeChange.
    *  Called by the framework when the application status pane
		*  size is changed.
    */

    void HandleStatusPaneSizeChange();

public:
  static TInt Start(TAny* appui);
  // MFrozenBubbleAction interface methods
  void FlipBackBuffer(void);
  void BitBlt(TInt iBltType, CExtendedBitmap &iBitmap, const TPoint &aPosition);
  void BitBlt(TInt iBltType, CExtendedBitmap &iBitmap, const TPoint &aPosition, const TRect &aRect);
  void BitBltBack(CPrintableExtendedBitmap &iBitmapTarget);
  void DoExit(void);
  virtual TBool ReadIniFile(TUid iInfo, void *ptr, TUint &size);
  virtual TBool CheckIniFile(TUid iInfo);
  virtual void WriteIniFile(TUid iInfo, void *ptr, TUint size);
  virtual void RemoveIniData(TUid iInfo);
  CFrozenBubbleMain& MainEngine();
private:
  /**
  * From CEikAppUi, handles key events.
  * @param aKeyEvent Event to handled.
  * @param aType Type of the key event. 
  * @return Response code (EKeyWasConsumed, EKeyWasNotConsumed). 
  */
  virtual TKeyResponse HandleKeyEventL(const TKeyEvent& aKeyEvent,TEventCode aType);

protected:
  void HandleForegroundEventL(TBool aForeground);
    
private: // Data

    /**
    * The application view
    * Owned by CFrozenBubbleAppUi
    */
    CFrozenBubbleAppView* iAppView;
    CFrozenBubbleMain* iMainEngine;
    CIdle *iLancher;
    TPoint ptTopLeft;        
};

#endif // __FROZENBUBBLEAPPUI_h__

// End of File
