/*
    This file is part of Frozen Bubble for S60.

    Frozen Bubble for S60 is free software; you can redistribute
    it and/or modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2 of
    the License, or (at your option) any later version.

    Frozen Bubble for S60 is distributed in the hope that it
    will be useful, but WITHOUT ANY WARRANTY; without even the implied
    warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
    See the GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Frozen Bubble for S60; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// INCLUDE FILES
#include <coemain.h>
#include <eikspane.h>
#include "FrozenBubble.hrh"
#include "FrozenBubbleAppUi.h"
#ifdef UIQ3
#include <qikappui.h>
#include <FrozenBubble.rsg>
#endif

#include "FrozenBubbleAppView.h"
// ============================ MEMBER FUNCTIONS ===============================

// -----------------------------------------------------------------------------
// CFrozenBubbleAppView::NewL()
// Two-phased constructor.
// -----------------------------------------------------------------------------
//
CFrozenBubbleAppView* CFrozenBubbleAppView::NewL( const TRect& aRect ) {
  CFrozenBubbleAppView* self = CFrozenBubbleAppView::NewLC( aRect );
  CleanupStack::Pop( self );
  return self;
}

// -----------------------------------------------------------------------------
// CFrozenBubbleAppView::NewLC()
// Two-phased constructor.
// -----------------------------------------------------------------------------
//
CFrozenBubbleAppView* CFrozenBubbleAppView::NewLC( const TRect& aRect ) {
#ifdef UIQ3
  CFrozenBubbleAppView* self = new ( ELeave ) CFrozenBubbleAppView(static_cast<CQikAppUi&>(*(CEikonEnv::Static()->EikAppUi())));
#else
  CFrozenBubbleAppView* self = new ( ELeave ) CFrozenBubbleAppView;
#endif
  CleanupStack::PushL( self );
  self->ConstructL( aRect );
  return self;
}

// -----------------------------------------------------------------------------
// CFrozenBubbleAppView::ConstructL()
// Symbian 2nd phase constructor can leave.
// -----------------------------------------------------------------------------
//
void CFrozenBubbleAppView::ConstructL( const TRect& aRect ) {
#ifdef UIQ3
  CQikViewBase::ConstructL();
#endif
  // Create a window for this application view
  CreateWindowL();

  // Set the windows size
  SetRect( aRect );

  SetExtentToWholeScreen();

  bmpBackBuffer = new ( ELeave ) CFbsBitmap();
  User::LeaveIfError(bmpBackBuffer->Create( iEikonEnv->ScreenDevice()->SizeInPixels(), iEikonEnv->DefaultDisplayMode() ));
  // Create bitmap device for the bitmap 
  iBackBufferDevice = CFbsBitmapDevice::NewL(bmpBackBuffer);
//  iBackBufferDevice->SetAutoUpdate(ETrue);
  // Create graphics context for the bitmap 
  User::LeaveIfError(iBackBufferDevice->CreateContext( iBackBufferGc ));

  DrawNow();
  
  iDirectScreenAccess = CDirectScreenAccess::NewL(iEikonEnv->WsSession(), *iEikonEnv->ScreenDevice(), Window(), *this);

  // Activate the window, which makes it ready to be drawn
  ActivateL();

  SetupDirectScreenAccessL();
  
  BlankScreen();

#ifdef UIQ3
  PreemptViewConstructionL();
  TQikViewMode viewmode;
  viewmode.SetFullscreen();
  SetViewModeL(viewmode);
#endif
}


#ifdef UIQ3
// -----------------------------------------------------------------------------
// CFrozenBubbleAppView::CFrozenBubbleAppView()
// C++ default constructor can NOT contain any code, that might leave.
// -----------------------------------------------------------------------------
//
CFrozenBubbleAppView::CFrozenBubbleAppView(CQikAppUi& aAppUi):CQikViewBase(aAppUi, KNullViewId) {
  // No implementation required
}

		
// from CQikViewBase
TVwsViewId CFrozenBubbleAppView::ViewId()const {
		return TVwsViewId(TUid::Uid(_UID3),TUid::Uid(1));
}

// from CQikViewBase
void CFrozenBubbleAppView::ViewConstructL() {
	ViewConstructFromResourceL(R_FB_UI_CONFIGURATIONS);
}

TKeyResponse CFrozenBubbleAppView::OfferKeyEventL(const TKeyEvent& aKeyEvent,TEventCode aType) {
	CQikViewBase::OfferKeyEventL(aKeyEvent, aType);
	return EKeyWasNotConsumed;
}

/**
 * HandlePointerEventL
 * @see CCoeControl
 * @param aPointerEvent, checked for button down and report this to the observer
 */
void CFrozenBubbleAppView::HandlePointerEventL( const TPointerEvent& aPointerEvent )
{
	if(aPointerEvent.iType == TPointerEvent::EButton1Up)
	{
		TRect selectRect(0,280,80,320);
		TRect exitRect(160,280,240,320);

		if(selectRect.Contains(aPointerEvent.iPosition))
		{
			static_cast<CFrozenBubbleAppUi&>(*(iEikonEnv->EikAppUi())).MainEngine().ExternalEvent(EVT_SELECT);
		}
		else if(exitRect.Contains(aPointerEvent.iPosition))
		{ 
			static_cast<CFrozenBubbleAppUi&>(*(iEikonEnv->EikAppUi())).MainEngine().ExternalEvent(EVT_ESCAPE);
		}
	}
}

#else
// -----------------------------------------------------------------------------
// CFrozenBubbleAppView::CFrozenBubbleAppView()
// C++ default constructor can NOT contain any code, that might leave.
// -----------------------------------------------------------------------------
//
CFrozenBubbleAppView::CFrozenBubbleAppView() {
  // No implementation required
}
#endif
// -----------------------------------------------------------------------------
// CFrozenBubbleAppView::~CFrozenBubbleAppView()
// Destructor.
// -----------------------------------------------------------------------------
//
CFrozenBubbleAppView::~CFrozenBubbleAppView() {
  delete iDirectScreenAccess;
  delete iBackBufferDevice;
  delete iBackBufferGc;
  delete bmpBackBuffer;
}


// -----------------------------------------------------------------------------
// CFrozenBubbleAppView::Draw()
// Draws the display.
// -----------------------------------------------------------------------------
//
void CFrozenBubbleAppView::Draw( const TRect& /*aRect*/ ) const {
  // Get the standard graphics context
  CWindowGc& gc = SystemGc();

  gc.BitBlt(TPoint(0, 0), bmpBackBuffer);
}

// ---------------------------------------------------------
// CFrozenBubbleContainer::HandleControlEventL(
//     CCoeControl* aControl,TCoeEvent aEventType)
// ---------------------------------------------------------
//
void CFrozenBubbleAppView::HandleControlEventL(CCoeControl* /*aControl*/,TCoeEvent /*aEventType*/) {
  //
}

// -----------------------------------------------------------------------------
// CFrozenBubbleAppView::SizeChanged()
// Called by framework when the view size is changed.
// -----------------------------------------------------------------------------
//
void CFrozenBubbleAppView::SizeChanged() {  
  DrawNow();
}

void CFrozenBubbleAppView::SetupDirectScreenAccessL(){
  iDirectEnabled = ETrue;

  // Initialise DSA
  iDirectScreenAccess->StartL();
  
//  iDirectScreenAccess->ScreenDevice()->SetAutoUpdate(ETrue);
  
  // Get graphics context for it
  iGc = iDirectScreenAccess->Gc();
  
  // Get region that DSA can draw in
  iRegion = iDirectScreenAccess->DrawingRegion();
  
  // Set the display to clip to this region
  iGc->SetClippingRegion(iRegion);
}

void CFrozenBubbleAppView::Restart(RDirectScreenAccess::TTerminationReasons /*aReason*/){
  SetupDirectScreenAccessL();
//  iDirectEnabled = ETrue;
}

void CFrozenBubbleAppView::AbortNow(RDirectScreenAccess::TTerminationReasons /*aReason*/){
  //fireIdle->Cancel();
  iDirectEnabled = EFalse;
}

void CFrozenBubbleAppView::FlipBackBuffer(TPoint aTopLeft, TRect aSrcRect){
  if (iDirectEnabled){
    iGc->BitBlt(aTopLeft, bmpBackBuffer, aSrcRect);

    iEikonEnv->WsSession().Flush();
    // Force screen update: this required for WINS, but may
    // not be for all hardware
    iDirectScreenAccess->ScreenDevice()->Update();
  }
}

void CFrozenBubbleAppView::BlankScreen(void){
  if (iDirectEnabled){
    iBackBufferGc->SetPenStyle( CGraphicsContext::ENullPen );
    iBackBufferGc->SetBrushColor( KRgbBlack );
    iBackBufferGc->SetBrushStyle( CGraphicsContext::ESolidBrush );
    iBackBufferGc->DrawRect( TRect(TPoint(0,0), bmpBackBuffer->SizeInPixels()) );
    iBackBufferGc->SetBrushStyle( CGraphicsContext::ENullBrush );
    iGc->BitBlt(TPoint(0, 0), bmpBackBuffer);
    iEikonEnv->WsSession().Flush();
    // Force screen update: this required for WINS, but may
    // not be for all hardware
    iDirectScreenAccess->ScreenDevice()->Update();
  }
}

// End of File
