/*
    This file is part of Frozen Bubble for S60.

    Frozen Bubble for S60 is free software; you can redistribute
    it and/or modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2 of
    the License, or (at your option) any later version.

    Frozen Bubble for S60 is distributed in the hope that it
    will be useful, but WITHOUT ANY WARRANTY; without even the implied
    warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
    See the GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Frozen Bubble for S60; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __FROZENBUBBLEAPPVIEW_h__
#define __FROZENBUBBLEAPPVIEW_h__

// INCLUDES
#include <coecntrl.h>
#include "CExtendedBitmap.h"

// CLASS DECLARATION
#ifdef UIQ3
#include <qikviewbase.h>
class CFrozenBubbleAppView : public CQikViewBase, MDirectScreenAccess {
#else
class CFrozenBubbleAppView : public CCoeControl, MCoeControlObserver, MDirectScreenAccess {
#endif
  public: // New methods

    /**
    * NewL.
    * Two-phased constructor.
    * Create a CFrozenBubbleAppView object, which will draw itself to aRect.
    * @param aRect The rectangle this view will be drawn to.
    * @return a pointer to the created instance of CFrozenBubbleAppView.
    */
    static CFrozenBubbleAppView* NewL( const TRect& aRect );

    /**
    * NewLC.
    * Two-phased constructor.
    * Create a CFrozenBubbleAppView object, which will draw itself
    * to aRect.
    * @param aRect Rectangle this view will be drawn to.
    * @return A pointer to the created instance of CFrozenBubbleAppView.
    */
    static CFrozenBubbleAppView* NewLC( const TRect& aRect );

    /**
    * ~CFrozenBubbleAppView
    * Virtual Destructor.
    */
    virtual ~CFrozenBubbleAppView();

  public:  // Functions from base classes

    /**
    * From CCoeControl, Draw
    * Draw this CFrozenBubbleAppView to the screen.
    * @param aRect the rectangle of this view that needs updating
    */
    void Draw( const TRect& aRect ) const;

    /**
    * From CoeControl, SizeChanged.
    * Called by framework when the view size is changed.
    */
    virtual void SizeChanged();

  	/**
  	* From MCoeControlObserver
  	* Acts upon changes in the hosted control's state. 
  	*
  	* @param	aControl	The control changing its state
  	* @param	aEventType	The type of control event 
  	*/
    void HandleControlEventL(CCoeControl* aControl,TCoeEvent aEventType);

    // Draw black screen
    void BlankScreen(void);

  public: // New functions
    void SetupDirectScreenAccessL();
    void Restart(RDirectScreenAccess::TTerminationReasons aReason);
    void AbortNow(RDirectScreenAccess::TTerminationReasons aReason);
    
    void FlipBackBuffer(TPoint aTopLeft, TRect aSrcRect);
    void BlitToBackBuffer(CExtendedBitmap &iBitmap, const TPoint &aPosition){
      iBitmap.BitBlt(iBackBufferGc, aPosition);
    }
    void BlitToBackBuffer(CExtendedBitmap &iBitmap, const TPoint &aPosition, const TRect &aRect){
      iBitmap.BitBlt(iBackBufferGc, aPosition, aRect);
    }
    void BlitToBackBufferMasked(CExtendedBitmap &iBitmap, const TPoint &aPosition){
      iBitmap.BitBltMasked(iBackBufferGc, aPosition);
    }
    void BlitToBackBufferMasked(CExtendedBitmap &iBitmap, const TPoint &aPosition, const TRect &aRect){
      iBitmap.BitBltMasked(iBackBufferGc, aPosition, aRect);
    }
    void BitBltBack(CPrintableExtendedBitmap &iBitmapTarget){
      iBitmapTarget.CopyFrom( *bmpBackBuffer );
    }

  private: // Constructors

    /**
    * ConstructL
    * 2nd phase constructor.
    * Perform the second phase construction of a
    * CFrozenBubbleAppView object.
    * @param aRect The rectangle this view will be drawn to.
    */
    void ConstructL(const TRect& aRect);
#ifdef UIQ3
    /**
    * CFrozenBubbleAppView.
    * C++ default constructor.
    */
    CFrozenBubbleAppView(CQikAppUi& aAppUi);
	
    // from CQikViewBase
     TVwsViewId ViewId()const;

	 // from CQikViewBase
	void ViewConstructL();

	TKeyResponse OfferKeyEventL(const TKeyEvent& aKeyEvent,TEventCode aType);

	/**
	 * HandlePointerEventL
	 * @see CCoeControl
	 * @param aPointerEvent, checked for button down and report this to the observer
	 */
	void HandlePointerEventL( const TPointerEvent& aPointerEvent );

#else
    /**
    * CFrozenBubbleAppView.
    * C++ default constructor.
    */
    CFrozenBubbleAppView();
#endif
  private: //data
    CDirectScreenAccess* iDirectScreenAccess;
    CFbsBitGc* iGc;
    RRegion* iRegion;
    CFbsBitmap *bmpBackBuffer;
    CFbsBitGc *iBackBufferGc;
    CFbsBitmapDevice *iBackBufferDevice;
    
    TBool iDirectEnabled;
};

#endif // __FROZENBUBBLEAPPVIEW_h__

// End of File
