/*
    This file is part of Frozen Bubble for S60.

    Frozen Bubble for S60 is free software; you can redistribute
    it and/or modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2 of
    the License, or (at your option) any later version.

    Frozen Bubble for S60 is distributed in the hope that it
    will be useful, but WITHOUT ANY WARRANTY; without even the implied
    warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
    See the GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Frozen Bubble for S60; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// INCLUDE FILES
#include "FrozenBubbleAppUi.h"
#include "FrozenBubbleDocument.h"

// ============================ MEMBER FUNCTIONS ===============================

// -----------------------------------------------------------------------------
// CFrozenBubbleDocument::NewL()
// Two-phased constructor.
// -----------------------------------------------------------------------------
//
CFrozenBubbleDocument* CFrozenBubbleDocument::NewL( CEikApplication& aApp ) {
  CFrozenBubbleDocument* self = NewLC( aApp );
  CleanupStack::Pop( self );
  return self;
}

// -----------------------------------------------------------------------------
// CFrozenBubbleDocument::NewLC()
// Two-phased constructor.
// -----------------------------------------------------------------------------
//
CFrozenBubbleDocument* CFrozenBubbleDocument::NewLC(CEikApplication& aApp) {
  CFrozenBubbleDocument* self =
      new ( ELeave ) CFrozenBubbleDocument( aApp );

  CleanupStack::PushL( self );
  self->ConstructL();
  return self;
}

// -----------------------------------------------------------------------------
// CFrozenBubbleDocument::ConstructL()
// Symbian 2nd phase constructor can leave.
// -----------------------------------------------------------------------------
//
void CFrozenBubbleDocument::ConstructL() {
  // No implementation required
}

// -----------------------------------------------------------------------------
// CFrozenBubbleDocument::CFrozenBubbleDocument()
// C++ default constructor can NOT contain any code, that might leave.
// -----------------------------------------------------------------------------
//

CFrozenBubbleDocument::CFrozenBubbleDocument( CEikApplication& aApp )
#ifdef UIQ3
   :  CQikDocument( aApp ) {
#else
    : CAknDocument( aApp ) {
#endif
  // No implementation required
}

// ---------------------------------------------------------------------------
// CFrozenBubbleDocument::~CFrozenBubbleDocument()
// Destructor.
// ---------------------------------------------------------------------------
//
CFrozenBubbleDocument::~CFrozenBubbleDocument() {
  // No implementation required
}

// ---------------------------------------------------------------------------
// CFrozenBubbleDocument::CreateAppUiL()
// Constructs CreateAppUi.
// ---------------------------------------------------------------------------
//
CEikAppUi* CFrozenBubbleDocument::CreateAppUiL() {
  // Create the application user interface, and return a pointer to it;
  // the framework takes ownership of this object
  return ( static_cast <CEikAppUi*> ( new ( ELeave )
                                      CFrozenBubbleAppUi ) );
}

// End of File
